(* Copyright 2009, Thant Tessman. Permission is granted for
   non-commercial use provided this copyright notice is preserved. All
   other rights reserved. *)

structure Main =
struct

fun checkFile (data_file, 
               type_file_opt,
               flatten_file_opt) =
    let
      val data = case type_file_opt of
		   SOME type_file =>
		   let
		     val ds = Path.stringToSymbol ("<" ^ data_file ^ ">")
		     val t = ParseDLT.parseFile type_file
		     val d = ParseDL.parseFile data_file
		   in
		     DL.REC [DL.BINDING (ds, t, d)]
		   end
		 | NONE => ParseDL.parseFile data_file

      val bind_errors = DL.findDuplicates data
      val ref_errors = DL.findRefs data
      val data' = DL.addIndirections (data, ref_errors)
      val data'' = DL.resolve (data')
      val type_errors = DL.checkTypes(data'')
    in
      (DL.dumpDuplicateErrors (TextIO.stdErr, bind_errors)
     ; DL.dumpRefErrors (TextIO.stdErr, ref_errors)
     ; DL.dumpTypeErrors (TextIO.stdErr, type_errors)
     ; (case flatten_file_opt of
          SOME file =>
          let
            val data''' = DL.unresolve(DL.flatten(data''))
            val out = TextIO.openOut(file)
          in
            DL.dumpValue (out, DL.DATA data''')
          end
        | NONE => ()))
    end

end

