(* Copyright 2009, Thant Tessman. Permission is granted for
   non-commercial use provided this copyright notice is preserved. All
   other rights reserved. *)

signature KEY =
sig
  type key
  val compare : (key * key) -> order
end

functor Dict (Key : KEY) =
struct

datatype 'v node 
  = LEAF 
  | BRANCH of Key.key * 'v * 'v node * 'v node

fun newDict () = LEAF

fun insert (LEAF, new_key, new_value)
    = BRANCH (new_key, new_value, LEAF, LEAF)

  | insert (BRANCH (key, value, left, right), new_key, new_value)
    = case Key.compare (new_key, key) of
        EQUAL => BRANCH (new_key, new_value, left, right)
      | LESS => BRANCH (key, value, insert(left, new_key, new_value), right)
      | GREATER => BRANCH (key, value, left, insert(right, new_key, new_value))

fun find (LEAF, _) = NONE
  | find (BRANCH (key, value, left, right), new_key)
    = case Key.compare (new_key, key) of
        EQUAL => SOME value
      | LESS => find (left, new_key)
      | GREATER => find (right, new_key)

fun toList dict =
    let
      fun doit (LEAF, list) = list
        | doit (BRANCH (key, value, left, right), list)
          = doit (left, (key, value) :: doit(right, list))
    in
      doit (dict, [])
    end

end

