(* Copyright 2009, Thant Tessman. Permission is granted for
   non-commercial use provided this copyright notice is preserved. All
   other rights reserved. *)

structure Tokens = Tokens

type pos = int
type svalue = Tokens.svalue
type ('a,'b) token = ('a,'b) Tokens.token
type lexresult = (svalue,pos) token

val ln = DL.lineno
val prefix = DL.prefix

val eof = fn () => Tokens.EOF(!ln, !ln)

val error = fn e => TextIO.output(TextIO.stdErr, 
				  !prefix
				  ^ "lex error line " 
				  ^ Int.toString(!ln) 
				  ^ ": " ^ e ^ "\n")

%%

%header (functor DLTLexerFun(structure Tokens: DLTGrammar_TOKENS));

alpha = [a-zA-Z_];
digit = [0-9];
id = {alpha}({alpha}|{digit})*;
symbol = #{id};

ws = [\ \t];
nat = [0-9]+;
real = (([0-9]+(\.[0-9]+)?)|(\.[0-9]+))([eE][+-]?[0-9]+)?;

vecn = "vec"{nat};

%%

"\n" 	=>	(ln := !ln + 1; lex());
{ws}+	=>	(lex());

"any"	=>	(Tokens.ANY_LIT(!ln, !ln));

"char"	=>	(Tokens.CHR_LIT(!ln, !ln));
"enum"	=>	(Tokens.ENUM_LIT(!ln, !ln));
"int"	=>	(Tokens.INT_LIT(!ln, !ln));
"real"	=>	(Tokens.REAL_LIT(!ln, !ln));
"sym"	=>	(Tokens.SYM_LIT(!ln, !ln));

"vec"	=>	(Tokens.VEC_LIT(!ln, !ln));

{vecn}	=>	(let
		     val s = String.extract(yytext, 3, NONE);
		 in
		     case Int.fromString(s) of
			 SOME i => Tokens.VECN_LIT(i, !ln, !ln)
		       | NONE => (error("bad vecN: " ^ yytext);
				  Tokens.VECN_LIT(0, !ln, !ln))
		 end);

"rec"	=>	(Tokens.REC_LIT(!ln, !ln));

"{"	=>	(Tokens.LEFT_CURLY(!ln, !ln));
"}"	=>	(Tokens.RIGHT_CURLY(!ln, !ln));

":"	=>	(Tokens.COLON(!ln, !ln));


{symbol} =>	(let
		     val s = String.extract(yytext, 1, NONE);
		 in
		     Tokens.SYM(Path.stringToSymbol(s), !ln, !ln)
		 end);

{id}	=>	(Tokens.NAME(Path.stringToSymbol(yytext), !ln, !ln));

.	=>	(error ("lex error: "^yytext); lex());
