(* Copyright 2009, Thant Tessman. Permission is granted for
   non-commercial use provided this copyright notice is preserved. All
   other rights reserved. *)

structure ParseDLT =
struct

structure GrammarLrVals 
  = DLTGrammarLrValsFun(structure Token = LrParser.Token)

structure Lexer = DLTLexerFun(structure Tokens = GrammarLrVals.Tokens)

structure Parser = 
Join(structure ParserData = GrammarLrVals.ParserData
     structure Lex = Lexer
     structure LrParser=LrParser)


fun parse (prefix, reader : int -> string) =
    let
      val lexer = Parser.makeLexer reader
      fun dump text = TextIO.output(TextIO.stdErr, text)
      fun dumpl text_list = List.app dump text_list

      fun printError (s, l, _) =
          dumpl [prefix, "parse error line: ", Int.toString(l), ": ", s, "\n"]
      val (result, _) = (DL.reset();
			 DL.prefix := prefix;
			 Parser.parse(0,lexer,printError,()))
    in
      result
    end

fun parseFile file =
    let
      val instream = TextIO.openIn(file)
      fun reader n = TextIO.inputN(instream, n)
      val prefix = file ^ ": "
    in
      parse (prefix, reader)
    end

fun parseString s =
    let
      val text = ref (s :: [])
      val reader : int -> string =
	  fn _ => case !text of
		    [] => ""
		  | first :: rest => (text := rest; first)
    in
      parse ("", reader)
    end

end

