(* Copyright 2009, Thant Tessman. Permission is granted for
   non-commercial use provided this copyright notice is preserved. All
   other rights reserved. *)

structure Path = 
struct

datatype sym = SYM of int;

fun sameSym(SYM a, SYM b) = a=b

fun compareSym(SYM a, SYM b) = Int.compare(a,b)

val symbols = ref (1, "" :: []);

fun reset () = symbols := (1, "" :: []);

datatype elem =
	 BINDING of sym
       | INDEX of int;

fun sameElem (BINDING a, BINDING b) = sameSym(a,b)
  | sameElem (INDEX a, INDEX b) = a=b
  | sameElem (_, _) = false

type path = elem list

fun samePath (a :: a_rest, b :: b_rest) = 
    sameElem(a,b) andalso samePath(a_rest, b_rest)
  | samePath ([], []) = true
  | samePath (_, _) = false

fun stringToSymbol name =
    let
	val (size, names) = !symbols;
	fun find (i, []) = (symbols := (size+1,name::names); SYM size)
	  | find (i, first::rest) =
	    case String.compare (name, first) of
		EQUAL => SYM i
	      | _ => find (i-1, rest)
    in
	find (size-1,names)
    end

fun symbolToString (SYM i) = 
    let
	val (size, names) = !symbols
    in
	List.nth (names, size-i-1)
    end


fun pathToString path =
    let
	fun f [] = ""
	  | f ((BINDING b)::rest) =
	    "." ^ (symbolToString b) ^ (f rest)
	  | f ((INDEX i)::rest) =
	    "[" ^ (Int.toString i) ^ "]" ^ (f rest)
    in
	case path of
	    [] => ""
	  | (BINDING b)::rest => (symbolToString b) ^ (f rest)
	  | (INDEX i)::rest => "[" ^ (Int.toString i) ^ "]" ^ (f rest)
    end

(* Returns b trimmed by whatever matches a. *)

fun trim ((BINDING a)::a_rest, path as (BINDING b)::b_rest) =
    if sameSym(a, b)
    then trim(a_rest, b_rest)
    else path
  | trim ((INDEX a)::a_rest, path as (INDEX b)::b_rest) =
    if a=b
    then trim(a_rest, b_rest)
    else path
  | trim (_, path) = path


end

