(* Copyright 2009, Thant Tessman. Permission is granted for
   non-commercial use provided this copyright notice is preserved. All
   other rights reserved. *)

structure Tree =
struct

datatype ('k, 'v) node = NODE of 'v option * ('k * ('k, 'v) node) list

val insert : (('k * 'k) -> bool) * 'k list * 'v * ('k, 'v) node
	     -> ('k, 'v) node
  = fn (f, path, value, tree : ('k, 'v) node) =>
       let

	 fun doBranches (key, rest, []) =
	     (key, doNode (rest, NODE (NONE, []))) :: []
	   | doBranches (key, rest, (k, n) :: branches) =
	     if f (key, k)
	     then (k, doNode (rest, n)) :: branches
	     else (k, n) :: (doBranches (key, rest, branches))

	 and doNode ([], NODE (_, branches)) = NODE (SOME value, branches)
	   | doNode (key :: rest, NODE (v, branches)) =
	     NODE (v, doBranches(key, rest, branches))

       in
	 doNode (path, tree)
       end

val find : (('k * 'k) -> bool) * 'k list * ('k, 'v) node
	   -> 'v option
  = fn (f, path, tree) =>
       let

	 fun doBranches (_, _, []) = NONE
           | doBranches (key, rest, (k, n) :: branches) =
             if f (key, k)
             then doNode (rest, n)
             else doBranches (key, rest, branches)

	 and doNode ([], NODE (v, _)) = v
	   | doNode (key :: rest, NODE (_, branches)) = 
             doBranches (key, rest, branches)
       in
         doNode (path, tree)
       end

end

